/* This is a PRECC parser for oberon2, based entirely on a Stephen Bevan
 * yacc script. It is not copied, in that I read every line of Stephen's
 * script and then changed everything to PRECC lingo - sort of like
 * translating from french to greek because there aren't exact translations
 * in all cases and I have to duplicate the semantics without the syntax,
 * and sometimes Stephen has had to work around a problem that exists
 * only in yacc so I have been able to write more cleanly. Nevertheless,
 * I have left all the original comments of Stephen's script in and added my
 * own where necessary.
 *
 * Peter T. Breuer 1993 (ptb@comlab.ox.ac.uk)
 *
 * This script (but not the comments) is Copyright P.T. Breuer 1993, but
 * you have my permission to use it as you like.
 *
 * This script uses the same lexer (lex, flex, whatever) as Stephen's so
 * comparisons between PRECC and yacc are possible. Indeed, I have made the
 * measurements.
 */

# define TOKEN int        /* token type gotten from lex  - PTB */

/* Use preccx2?.lib under MSDOS - PTB */

# define BEGIN call_mode=1;if (p_argc>1) freopen(p_argv[1],"rt",stdin);

# include "cc.h"
# include "oberonp2.h"

/* (don't need fancy stuff - PTB) */

/* Stephen says .... */



/* -*- Fundamental -*- */
/* 
** $Id: oberon_2_p.y,v 1.30 1992/06/26 09:07:30 bevan Exp $
*/
/*+file-overview
**
** A YACC/BISON definition for Oberon-2
**
** This is based on the definition given in :-
**
**    The Programming Language Oberon-2
**    H. M\"{o}ssenb\"{o}ck, N. Wirth
**    Institut f\"{u}r Computersysteme, ETH Z\"{u}rich
**    January 1992
**
**-file-overview
**+copyright
**
** Copyright (C) 1991 Department of Computer Science, University of Manchester
** All rights reserved.
**
**-copyright
**+author
**
** Stephen J. Bevan <bevan@cs.man.ac.uk> 19911216
**
**-author
**
** Incompatibilities with real Oberon-2 definition
** ===============================================
**
** Ambiguity problems
** ------------------
**
** Procedure Calls
**
** Oberon-2 is yet another in the long line on languages whose grammar
** is inherently ambiguous.  (This term comes from "Compiler Construction
** for Digital Computers", David Gries, Wiley International, 1971.  I
** haven't found a discussion of this in a more recent compiler book)
**
** An example of this is the procedure call statement.  From the grammar
** as defined in the technical report, the following :-
**
**   a(b)
** 
** could be either a procedure "a" with argument "b" or a procedure "a"
** with a type guard "b".  From the syntax, there is no way to tell them
** apart.  Resolution of this requires extra "semantic" information.
**
** The solution adopted here is to relax the syntax wherever necessary
** and punt the resolution to the "semantic phase".  In the application
** this grammar was written for, this phase is totally separate, but
** the usual solution is to embed some actions directly into the grammar
** to resolve the conflict at parse time, probably by using information
** stored in a symbol table.  If you want this solution, you'll have 
** to write it yourself. 
**
** [ an aside :
**
**  Personally I think that any language that requires symbol table
**  queries to guide parsing is flawed.  I think a better solution
**  is to make the syntax unambiguous.  For example this could
**  be done in Oberon-2 by using a different syntax for the guard
**  type e.g. instead of :-
**
**    a(b)(1,2)
**
**  something like :-
**
**    a<b>(1,2)
**
** ]
**
** Note that the grammar, as defined here, would allow something like :-
**
**   a(b,c)(d,e)
**
** which is totally bogus.  This might seem a bit liberal,
** but the original grammar isn't perfect as it would allow things like :-
**
**   a(b)(c)(d)
**
** which is also bogus.  If you can come up with a "syntax only"
** solution to this, I'd like to hear about it.
**
**
** Qualified Idents
**
** The syntax for referencing a value in another module and for
** referencing a field of a record is the same.  Consequently when a
** fragment such as "a.b" is met, it impossible to decide between the
** two on purely syntactically.  This wouldn't really matter except that
** the ETH grammar attempts to distinguish between module and record 
** references, however it is impossible to make this distinction 
** using a CFG.
**
** The most common way to decide if this is a qualified reference or
** a record designator is to examine the symbol table for "a".  If 
** "a" refers to a module, then "a.b" is a qualified reference, if it is
** a variable, then it is a record reference.
**
** This problem could have been avoided by using different syntax
** for module references.  Possible alternatives are a$b (Clu) or
** a!b (Mesa?)
**
** However, as soon as it is possible to decide which value is required
** based on the access method, it makes it possible to allow the same
** name to be bound to two or more different values.  Wirth's languages
** usually have one single name scope, so this would seem to go against
** the style.  However, Oberon-2 already breaks this rule, by allowing
** overloading of procedure names with resolution based on the receiever
** type.
**
** Still on Qualified Idents ...
**
** In a number of places in the following a Qualified_Ident has been 
** replaced by Type_Reference which is just a synonym for Qualified_Ident.
** The reasons for doing this is that in these cases, the Qualified_Ident
** should be referencing a Type (qualified or not) and as such it makes
** sense to make this intention clear in the syntax (even if it can only
** be enforced using context conditions).
*/

/* ... end of what Stephen said - PTB */

MAIN(Module)

# undef BEGIN
# undef CHAR
# undef END
# undef EXIT


static STATUS hid0(){
return p_exactly0(L_ARRAY);
}
STATUS ARRAY(){
PARSER hid0;
return((TOPARSER hid0)());
}

static STATUS hid1(){
return p_exactly0(L_BEGIN);
}
STATUS BEGIN(){
PARSER hid1;
return((TOPARSER hid1)());
}

static STATUS hid2(){
return p_exactly0(L_BY);
}
STATUS BY(){
PARSER hid2;
return((TOPARSER hid2)());
}

static STATUS hid3(){
return p_exactly0(L_CASE);
}
STATUS CASE(){
PARSER hid3;
return((TOPARSER hid3)());
}

static STATUS hid4(){
return p_exactly0(L_CONST);
}
STATUS CONST(){
PARSER hid4;
return((TOPARSER hid4)());
}

static STATUS hid5(){
return p_exactly0(L_DIV);
}
STATUS DIV(){
PARSER hid5;
return((TOPARSER hid5)());
}

static STATUS hid6(){
return p_exactly0(L_DO);
}
STATUS DO(){
PARSER hid6;
return((TOPARSER hid6)());
}

static STATUS hid7(){
return p_exactly0(L_ELSE);
}
STATUS ELSE(){
PARSER hid7;
return((TOPARSER hid7)());
}

static STATUS hid8(){
return p_exactly0(L_ELSIF);
}
STATUS ELSIF(){
PARSER hid8;
return((TOPARSER hid8)());
}

static STATUS hid9(){
return p_exactly0(L_END);
}
STATUS END(){
PARSER hid9;
return((TOPARSER hid9)());
}

static STATUS hid10(){
return p_exactly0(L_EXIT);
}
STATUS EXIT(){
PARSER hid10;
return((TOPARSER hid10)());
}

static STATUS hid11(){
return p_exactly0(L_FOR);
}
STATUS FOR(){
PARSER hid11;
return((TOPARSER hid11)());
}

static STATUS hid12(){
return p_exactly0(L_IMPORT);
}
STATUS IMPORT(){
PARSER hid12;
return((TOPARSER hid12)());
}

static STATUS hid13(){
return p_exactly0(L_IF);
}
STATUS IF(){
PARSER hid13;
return((TOPARSER hid13)());
}

static STATUS hid14(){
return p_exactly0(L_IN);
}
STATUS IN(){
PARSER hid14;
return((TOPARSER hid14)());
}

static STATUS hid15(){
return p_exactly0(L_IS);
}
STATUS IS(){
PARSER hid15;
return((TOPARSER hid15)());
}

static STATUS hid16(){
return p_exactly0(L_LOOP);
}
STATUS LOOP(){
PARSER hid16;
return((TOPARSER hid16)());
}

static STATUS hid17(){
return p_exactly0(L_MODULE);
}
STATUS MODULE(){
PARSER hid17;
return((TOPARSER hid17)());
}

static STATUS hid18(){
return p_exactly0(L_MOD);
}
STATUS MOD(){
PARSER hid18;
return((TOPARSER hid18)());
}

static STATUS hid19(){
return p_exactly0(L_NIL);
}
STATUS NIL(){
PARSER hid19;
return((TOPARSER hid19)());
}

static STATUS hid20(){
return p_exactly0(L_OF);
}
STATUS OF(){
PARSER hid20;
return((TOPARSER hid20)());
}

static STATUS hid21(){
return p_exactly0(L_OR);
}
STATUS OR(){
PARSER hid21;
return((TOPARSER hid21)());
}

static STATUS hid22(){
return p_exactly0(L_POINTER);
}
STATUS POINTER(){
PARSER hid22;
return((TOPARSER hid22)());
}

static STATUS hid23(){
return p_exactly0(L_PROCEDURE);
}
STATUS PROCEDURE(){
PARSER hid23;
return((TOPARSER hid23)());
}

static STATUS hid24(){
return p_exactly0(L_RECORD);
}
STATUS RECORD(){
PARSER hid24;
return((TOPARSER hid24)());
}

static STATUS hid25(){
return p_exactly0(L_REPEAT);
}
STATUS REPEAT(){
PARSER hid25;
return((TOPARSER hid25)());
}

static STATUS hid26(){
return p_exactly0(L_RETURN);
}
STATUS RETURN(){
PARSER hid26;
return((TOPARSER hid26)());
}

static STATUS hid27(){
return p_exactly0(L_THEN);
}
STATUS THEN(){
PARSER hid27;
return((TOPARSER hid27)());
}

static STATUS hid28(){
return p_exactly0(L_TO);
}
STATUS TO(){
PARSER hid28;
return((TOPARSER hid28)());
}

static STATUS hid29(){
return p_exactly0(L_TYPE);
}
STATUS TYPE(){
PARSER hid29;
return((TOPARSER hid29)());
}

static STATUS hid30(){
return p_exactly0(L_UNTIL);
}
STATUS UNTIL(){
PARSER hid30;
return((TOPARSER hid30)());
}

static STATUS hid31(){
return p_exactly0(L_VAR);
}
STATUS VAR(){
PARSER hid31;
return((TOPARSER hid31)());
}

static STATUS hid32(){
return p_exactly0(L_WHILE);
}
STATUS WHILE(){
PARSER hid32;
return((TOPARSER hid32)());
}

static STATUS hid33(){
return p_exactly0(L_WITH);
}
STATUS WITH(){
PARSER hid33;
return((TOPARSER hid33)());
}




static STATUS hid34(){
return p_exactly0(L_AMPERSAND);
}
STATUS AMPERSAND(){
PARSER hid34;
return((TOPARSER hid34)());
}

static STATUS hid35(){
return p_exactly0(L_CARET);
}
STATUS CARET(){
PARSER hid35;
return((TOPARSER hid35)());
}

static STATUS hid36(){
return p_exactly0(L_CHAR);
}
STATUS CHAR(){
PARSER hid36;
return((TOPARSER hid36)());
}

static STATUS hid37(){
return p_exactly0(L_CHAR_OR_STRING);
}
STATUS CHAR_OR_STRING(){
PARSER hid37;
return((TOPARSER hid37)());
}

static STATUS hid38(){
return p_exactly0(L_CLOSE_BRACE);
}
STATUS CLOSE_BRACE(){
PARSER hid38;
return((TOPARSER hid38)());
}

static STATUS hid39(){
return p_exactly0(L_CLOSE_BRACKET);
}
STATUS CLOSE_BRACKET(){
PARSER hid39;
return((TOPARSER hid39)());
}

static STATUS hid40(){
return p_exactly0(L_CLOSE_PAREN);
}
STATUS CLOSE_PAREN(){
PARSER hid40;
return((TOPARSER hid40)());
}

static STATUS hid41(){
return p_exactly0(L_COLON);
}
STATUS COLON(){
PARSER hid41;
return((TOPARSER hid41)());
}

static STATUS hid42(){
return p_exactly0(L_COLON_EQUALS);
}
STATUS COLON_EQUALS(){
PARSER hid42;
return((TOPARSER hid42)());
}

static STATUS hid43(){
return p_exactly0(L_COMMA);
}
STATUS COMMA(){
PARSER hid43;
return((TOPARSER hid43)());
}

static STATUS hid44(){
return p_exactly0(L_EQUALS);
}
STATUS EQUALS(){
PARSER hid44;
return((TOPARSER hid44)());
}

static STATUS hid45(){
return p_exactly0(L_FORWARD_SLASH);
}
STATUS FORWARD_SLASH(){
PARSER hid45;
return((TOPARSER hid45)());
}

static STATUS hid46(){
return p_exactly0(L_HASH);
}
STATUS HASH(){
PARSER hid46;
return((TOPARSER hid46)());
}

static STATUS hid47(){
return p_exactly0(L_HYPHEN);
}
STATUS HYPHEN(){
PARSER hid47;
return((TOPARSER hid47)());
}

static STATUS hid48(){
return p_exactly0(L_IDENTIFIER);
}
STATUS IDENTIFIER(){
PARSER hid48;
return((TOPARSER hid48)());
}

static STATUS hid49(){
return p_exactly0(L_INTEGER);
}
STATUS INTEGER(){
PARSER hid49;
return((TOPARSER hid49)());
}

static STATUS hid50(){
return p_exactly0(L_LEFT_CHEVRON);
}
STATUS LEFT_CHEVRON(){
PARSER hid50;
return((TOPARSER hid50)());
}

static STATUS hid51(){
return p_exactly0(L_LEFT_CHEVRON_EQUALS);
}
STATUS LEFT_CHEVRON_EQUALS(){
PARSER hid51;
return((TOPARSER hid51)());
}

static STATUS hid52(){
return p_exactly0(L_OPEN_BRACE);
}
STATUS OPEN_BRACE(){
PARSER hid52;
return((TOPARSER hid52)());
}

static STATUS hid53(){
return p_exactly0(L_OPEN_BRACKET);
}
STATUS OPEN_BRACKET(){
PARSER hid53;
return((TOPARSER hid53)());
}

static STATUS hid54(){
return p_exactly0(L_OPEN_PAREN);
}
STATUS OPEN_PAREN(){
PARSER hid54;
return((TOPARSER hid54)());
}

static STATUS hid55(){
return p_exactly0(L_PERIOD);
}
STATUS PERIOD(){
PARSER hid55;
return((TOPARSER hid55)());
}

static STATUS hid56(){
return p_exactly0(L_PERIOD_PERIOD);
}
STATUS PERIOD_PERIOD(){
PARSER hid56;
return((TOPARSER hid56)());
}

static STATUS hid57(){
return p_exactly0(L_PLUS);
}
STATUS PLUS(){
PARSER hid57;
return((TOPARSER hid57)());
}

static STATUS hid58(){
return p_exactly0(L_REAL);
}
STATUS REAL(){
PARSER hid58;
return((TOPARSER hid58)());
}

static STATUS hid59(){
return p_exactly0(L_RIGHT_CHEVRON);
}
STATUS RIGHT_CHEVRON(){
PARSER hid59;
return((TOPARSER hid59)());
}

static STATUS hid60(){
return p_exactly0(L_RIGHT_CHEVRON_EQUALS);
}
STATUS RIGHT_CHEVRON_EQUALS(){
PARSER hid60;
return((TOPARSER hid60)());
}

static STATUS hid61(){
return p_exactly0(L_SEMI_COLON);
}
STATUS SEMI_COLON(){
PARSER hid61;
return((TOPARSER hid61)());
}

static STATUS hid62(){
return p_exactly0(L_STAR);
}
STATUS STAR(){
PARSER hid62;
return((TOPARSER hid62)());
}

static STATUS hid63(){
return p_exactly0(L_STRING);
}
STATUS STRING(){
PARSER hid63;
return((TOPARSER hid63)());
}

static STATUS hid64(){
return p_exactly0(L_TILDE);
}
STATUS TILDE(){
PARSER hid64;
return((TOPARSER hid64)());
}

static STATUS hid65(){
return p_exactly0(L_VERTICAL_BAR);
}
STATUS VERTICAL_BAR(){
PARSER hid65;
return((TOPARSER hid65)());
}






static STATUS hid66 (){
PARSER Number_Difference, Boolean_Disjunction;
return p_orparse0n (TOPARSER Number_Difference,0,TOPARSER Boolean_Disjunction,0);
}
static STATUS hid67 (){
PARSER Number_Sum, hid66;
return p_orparse0n (TOPARSER Number_Sum,0,TOPARSER hid66,0);
}
STATUS Add_Op(){
PARSER hid67;
return((TOPARSER hid67)());
}



static STATUS hid68(){
PARSER COLON_EQUALS, Expression;
return p_andparse0n (TOPARSER COLON_EQUALS,0,TOPARSER Expression,0);
}
static STATUS hid69(){
PARSER Designator, hid68;
return p_andparse0n (TOPARSER Designator,0,TOPARSER hid68,0);
}
STATUS Assignment(){
PARSER hid69;
return((TOPARSER hid69)());
}


STATUS Boolean_Disjunction(){
PARSER OR;
return((TOPARSER OR)());
}



static STATUS hid70(){
PARSER Optional_Else, Case_End;
return p_andparse0n (TOPARSER Optional_Else,0,TOPARSER Case_End,0);
}
static STATUS hid71(){
PARSER Case_Items, hid70;
return p_andparse0n (TOPARSER Case_Items,0,TOPARSER hid70,0);
}
static STATUS hid72(){
PARSER OF, hid71;
return p_andparse0n (TOPARSER OF,0,TOPARSER hid71,0);
}
static STATUS hid73(){
PARSER Expression, hid72;
return p_andparse0n (TOPARSER Expression,0,TOPARSER hid72,0);
}
static STATUS hid74(){
PARSER Case_Start, hid73;
return p_andparse0n (TOPARSER Case_Start,0,TOPARSER hid73,0);
}
STATUS Case(){
PARSER hid74;
return((TOPARSER hid74)());
}


STATUS Case_End(){
PARSER END;
return((TOPARSER END)());
}


static STATUS hid75(){
PARSER Case_Item_Separator, Statement_Sequence;
return p_andparse0n (TOPARSER Case_Item_Separator,0,TOPARSER Statement_Sequence,0);
}
static STATUS hid76(){
PARSER Case_Labels_Sequence, hid75;
return p_andparse0n (TOPARSER Case_Labels_Sequence,0,TOPARSER hid75,0);
}
static STATUS hid77(){
PARSER hid76;
return p_option0n(TOPARSER hid76,0);
}
STATUS Case_Item(){
PARSER hid77;
return((TOPARSER hid77)());
}


STATUS Case_Item_Separator(){
PARSER COLON;
return((TOPARSER COLON)());
}


static STATUS hid78(){
PARSER Case_Items_Separator, Case_Item;
return p_andparse0n (TOPARSER Case_Items_Separator,0,TOPARSER Case_Item,0);
}
static STATUS hid79(){
PARSER hid78;
return p_many0n(TOPARSER hid78,0);
}
static STATUS hid80(){
PARSER Case_Item, hid79;
return p_andparse0n (TOPARSER Case_Item,0,TOPARSER hid79,0);
}
STATUS Case_Items(){
PARSER hid80;
return((TOPARSER hid80)());
}


STATUS Case_Items_Separator(){
PARSER VERTICAL_BAR;
return((TOPARSER VERTICAL_BAR)());
}


STATUS Case_Label(){
PARSER Constant_Expression;
return((TOPARSER Constant_Expression)());
}


STATUS Case_Label_Separator(){
PARSER PERIOD_PERIOD;
return((TOPARSER PERIOD_PERIOD)());
}


static STATUS hid81(){
PARSER Case_Label_Separator, Case_Label;
return p_andparse0n (TOPARSER Case_Label_Separator,0,TOPARSER Case_Label,0);
}
static STATUS hid82(){
PARSER hid81;
return p_option0n(TOPARSER hid81,0);
}
static STATUS hid83(){
PARSER Case_Label, hid82;
return p_andparse0n (TOPARSER Case_Label,0,TOPARSER hid82,0);
}
STATUS Case_Labels(){
PARSER hid83;
return((TOPARSER hid83)());
}


static STATUS hid84(){
PARSER Case_Labels_Sequence_Separator, Case_Labels;
return p_andparse0n (TOPARSER Case_Labels_Sequence_Separator,0,TOPARSER Case_Labels,0);
}
static STATUS hid85(){
PARSER hid84;
return p_many0n(TOPARSER hid84,0);
}
static STATUS hid86(){
PARSER Case_Labels, hid85;
return p_andparse0n (TOPARSER Case_Labels,0,TOPARSER hid85,0);
}
STATUS Case_Labels_Sequence(){
PARSER hid86;
return((TOPARSER hid86)());
}


STATUS Case_Labels_Sequence_Separator(){
PARSER COMMA;
return((TOPARSER COMMA)());
}


STATUS Case_Start(){
PARSER CASE;
return((TOPARSER CASE)());
}




STATUS Constant_Assignment(){
PARSER EQUALS;
return((TOPARSER EQUALS)());
}


static STATUS hid87(){
PARSER Constant_Expression, Constant_Declaration_Terminator;
return p_andparse0n (TOPARSER Constant_Expression,0,TOPARSER Constant_Declaration_Terminator,0);
}
static STATUS hid88(){
PARSER Constant_Assignment, hid87;
return p_andparse0n (TOPARSER Constant_Assignment,0,TOPARSER hid87,0);
}
static STATUS hid89(){
PARSER Ident_Def, hid88;
return p_andparse0n (TOPARSER Ident_Def,0,TOPARSER hid88,0);
}
STATUS Constant_Declaration(){
PARSER hid89;
return((TOPARSER hid89)());
}


static STATUS hid90(){
PARSER Constant_Declaration;
return p_many0n(TOPARSER Constant_Declaration,0);
}
STATUS Constant_Declarations(){
PARSER hid90;
return((TOPARSER hid90)());
}


STATUS Constant_Declaration_Terminator(){
PARSER SEMI_COLON;
return((TOPARSER SEMI_COLON)());
}



STATUS Constant_Expression(){
PARSER Expression;
return((TOPARSER Expression)());
}


static STATUS hid91(){
PARSER CONST, Constant_Declarations;
return p_andparse0n (TOPARSER CONST,0,TOPARSER Constant_Declarations,0);
}
static STATUS hid92(){
PARSER TYPE, Type_Declarations;
return p_andparse0n (TOPARSER TYPE,0,TOPARSER Type_Declarations,0);
}
static STATUS hid93(){
PARSER VAR, Var_Declarations;
return p_andparse0n (TOPARSER VAR,0,TOPARSER Var_Declarations,0);
}
static STATUS hid94 (){
PARSER hid92, hid93;
return p_orparse0n (TOPARSER hid92,0,TOPARSER hid93,0);
}
static STATUS hid95 (){
PARSER hid91, hid94;
return p_orparse0n (TOPARSER hid91,0,TOPARSER hid94,0);
}
STATUS Declaration_Simple_Item(){
PARSER hid95;
return((TOPARSER hid95)());
}


static STATUS hid96(){
PARSER Declaration_Simple_Item;
return p_many0n(TOPARSER Declaration_Simple_Item,0);
}
STATUS Declaration_Simple_Items(){
PARSER hid96;
return((TOPARSER hid96)());
}


static STATUS hid97 (){
PARSER Procedure_Declaration, Forward_Declaration;
return p_orparse0n (TOPARSER Procedure_Declaration,0,TOPARSER Forward_Declaration,0);
}
STATUS Declaration_Procedure(){
PARSER hid97;
return((TOPARSER hid97)());
}


static STATUS hid98(){
PARSER Declaration_Procedure;
return p_many0n(TOPARSER Declaration_Procedure,0);
}
STATUS Declaration_Procedures(){
PARSER hid98;
return((TOPARSER hid98)());
}


STATUS Declaration_Procedure_Terminator(){
PARSER SEMI_COLON;
return((TOPARSER SEMI_COLON)());
}


static STATUS hid99(){
PARSER Declaration_Simple_Items, Declaration_Procedures;
return p_andparse0n (TOPARSER Declaration_Simple_Items,0,TOPARSER Declaration_Procedures,0);
}
STATUS Declaration_Sequence(){
PARSER hid99;
return((TOPARSER hid99)());
}

/* Stephen says ... */

/*
** Designator has been written in a different way from the ETH
** definition to allow the abstract syntaxt tree to be built correctly
** using synthesised attributes.  (The ETH definition is good for
** inherited attributes which are difficult in a L[AL]R system.)
*/

/* End of what Stephen said - PTB */

static STATUS hid100 (){
PARSER Designator_Indirection, Designator_Guard;
return p_orparse0n (TOPARSER Designator_Indirection,0,TOPARSER Designator_Guard,0);
}
static STATUS hid101 (){
PARSER Designator_Array_Ref, hid100;
return p_orparse0n (TOPARSER Designator_Array_Ref,0,TOPARSER hid100,0);
}
static STATUS hid102 (){
PARSER Designator_Record_Ref, hid101;
return p_orparse0n (TOPARSER Designator_Record_Ref,0,TOPARSER hid101,0);
}
static STATUS hid103(){
PARSER hid102;
return p_option0n(TOPARSER hid102,0);
}
static STATUS hid104(){
PARSER Qualified_Ident, hid103;
return p_andparse0n (TOPARSER Qualified_Ident,0,TOPARSER hid103,0);
}
STATUS Designator_Base(){
PARSER hid104;
return((TOPARSER hid104)());
}


static STATUS hid105 (){
PARSER Designator_Indirection, Designator_Guard;
return p_orparse0n (TOPARSER Designator_Indirection,0,TOPARSER Designator_Guard,0);
}
static STATUS hid106 (){
PARSER Designator_Array_Ref, hid105;
return p_orparse0n (TOPARSER Designator_Array_Ref,0,TOPARSER hid105,0);
}
static STATUS hid107 (){
PARSER Designator_Record_Ref, hid106;
return p_orparse0n (TOPARSER Designator_Record_Ref,0,TOPARSER hid106,0);
}
static STATUS hid108(){
PARSER hid107;
return p_option0n(TOPARSER hid107,0);
}
static STATUS hid109(){
PARSER Designator_Base, hid108;
return p_andparse0n (TOPARSER Designator_Base,0,TOPARSER hid108,0);
}
STATUS Designator(){
PARSER hid109;
return((TOPARSER hid109)());
}



static STATUS hid110(){
PARSER Expression_List, Designator_Array_Ref_End;
return p_andparse0n (TOPARSER Expression_List,0,TOPARSER Designator_Array_Ref_End,0);
}
static STATUS hid111(){
PARSER Designator_Array_Ref_Start, hid110;
return p_andparse0n (TOPARSER Designator_Array_Ref_Start,0,TOPARSER hid110,0);
}
STATUS Designator_Array_Ref(){
PARSER hid111;
return((TOPARSER hid111)());
}


STATUS Designator_Array_Ref_End(){
PARSER CLOSE_BRACKET;
return((TOPARSER CLOSE_BRACKET)());
}

STATUS Designator_Array_Ref_Start(){
PARSER OPEN_BRACKET;
return((TOPARSER OPEN_BRACKET)());
}

/* Stephen says ... */


/*
** This is where this grammar deviates badly from the ETH definition.
** According to it, only Qualified_Idents should be valid between the
** parens, but this if this is done, it buggers up the 
** procedure/function call syntax.
*/

/* End of what Stephen said - PTB */


static STATUS hid112(){
PARSER Optional_Expression_List, CLOSE_PAREN;
return p_andparse0n (TOPARSER Optional_Expression_List,0,TOPARSER CLOSE_PAREN,0);
}
static STATUS hid113(){
PARSER OPEN_PAREN, hid112;
return p_andparse0n (TOPARSER OPEN_PAREN,0,TOPARSER hid112,0);
}
STATUS Designator_Guard(){
PARSER hid113;
return((TOPARSER hid113)());
}

static STATUS hid114(){
PARSER Expression_List;
return p_option0n(TOPARSER Expression_List,0);
}
STATUS Optional_Expression_List(){
PARSER hid114;
return((TOPARSER hid114)());
}

STATUS Designator_Indirection(){
PARSER CARET;
return((TOPARSER CARET)());
}



static STATUS hid115(){
PARSER PERIOD, Ident;
return p_andparse0n (TOPARSER PERIOD,0,TOPARSER Ident,0);
}
STATUS Designator_Record_Ref(){
PARSER hid115;
return((TOPARSER hid115)());
}




STATUS Exit(){
PARSER EXIT;
return((TOPARSER EXIT)());
}




static STATUS hid116(){
PARSER Simple_Expression, Expression_Optional_Relation_Simple_Expression;
return p_andparse0n (TOPARSER Simple_Expression,0,TOPARSER Expression_Optional_Relation_Simple_Expression,0);
}
STATUS Expression(){
PARSER hid116;
return((TOPARSER hid116)());
}


static STATUS hid117(){
PARSER Expression_List_Separator, Expression;
return p_andparse0n (TOPARSER Expression_List_Separator,0,TOPARSER Expression,0);
}
static STATUS hid118(){
PARSER hid117;
return p_many0n(TOPARSER hid117,0);
}
static STATUS hid119(){
PARSER Expression, hid118;
return p_andparse0n (TOPARSER Expression,0,TOPARSER hid118,0);
}
STATUS Expression_List(){
PARSER hid119;
return((TOPARSER hid119)());
}


STATUS Expression_List_Separator(){
PARSER COMMA;
return((TOPARSER COMMA)());
}


static STATUS hid120(){
PARSER Relation, Simple_Expression;
return p_andparse0n (TOPARSER Relation,0,TOPARSER Simple_Expression,0);
}
static STATUS hid121(){
PARSER hid120;
return p_option0n(TOPARSER hid120,0);
}
STATUS Expression_Optional_Relation_Simple_Expression(){
PARSER hid121;
return((TOPARSER hid121)());
}



static STATUS hid122(){
PARSER Expression, CLOSE_PAREN;
return p_andparse0n (TOPARSER Expression,0,TOPARSER CLOSE_PAREN,0);
}
static STATUS hid123(){
PARSER OPEN_PAREN, hid122;
return p_andparse0n (TOPARSER OPEN_PAREN,0,TOPARSER hid122,0);
}
static STATUS hid124 (){
PARSER hid123, Factor_Negative;
return p_orparse0n (TOPARSER hid123,0,TOPARSER Factor_Negative,0);
}
static STATUS hid125 (){
PARSER Factor_Set, hid124;
return p_orparse0n (TOPARSER Factor_Set,0,TOPARSER hid124,0);
}
static STATUS hid126 (){
PARSER Factor_Nil, hid125;
return p_orparse0n (TOPARSER Factor_Nil,0,TOPARSER hid125,0);
}
static STATUS hid127 (){
PARSER Factor_Char_Or_String, hid126;
return p_orparse0n (TOPARSER Factor_Char_Or_String,0,TOPARSER hid126,0);
}
static STATUS hid128 (){
PARSER Factor_String, hid127;
return p_orparse0n (TOPARSER Factor_String,0,TOPARSER hid127,0);
}
static STATUS hid129 (){
PARSER Factor_Char, hid128;
return p_orparse0n (TOPARSER Factor_Char,0,TOPARSER hid128,0);
}
static STATUS hid130 (){
PARSER Factor_Number, hid129;
return p_orparse0n (TOPARSER Factor_Number,0,TOPARSER hid129,0);
}
static STATUS hid131 (){
PARSER Factor_Designator, hid130;
return p_orparse0n (TOPARSER Factor_Designator,0,TOPARSER hid130,0);
}
STATUS Factor(){
PARSER hid131;
return((TOPARSER hid131)());
}

STATUS Factor_Designator(){
PARSER Procedure_Call;
return((TOPARSER Procedure_Call)());
}

STATUS Factor_Char(){
PARSER CHAR;
return((TOPARSER CHAR)());
}

STATUS Factor_Char_Or_String(){
PARSER CHAR_OR_STRING;
return((TOPARSER CHAR_OR_STRING)());
}

static STATUS hid132(){
PARSER Factor_Logical_Negation, Factor;
return p_andparse0n (TOPARSER Factor_Logical_Negation,0,TOPARSER Factor,0);
}
STATUS Factor_Negative(){
PARSER hid132;
return((TOPARSER hid132)());
}

STATUS Factor_Logical_Negation(){
PARSER TILDE;
return((TOPARSER TILDE)());
}

STATUS Factor_Nil(){
PARSER NIL;
return((TOPARSER NIL)());
}

static STATUS hid133 (){
PARSER REAL, INTEGER;
return p_orparse0n (TOPARSER REAL,0,TOPARSER INTEGER,0);
}
STATUS Factor_Number(){
PARSER hid133;
return((TOPARSER hid133)());
}

STATUS Factor_Set(){
PARSER Set;
return((TOPARSER Set)());
}

STATUS Factor_String(){
PARSER STRING;
return((TOPARSER STRING)());
}




static STATUS hid134(){
PARSER Statement_Sequence, For_End;
return p_andparse0n (TOPARSER Statement_Sequence,0,TOPARSER For_End,0);
}
static STATUS hid135(){
PARSER DO, hid134;
return p_andparse0n (TOPARSER DO,0,TOPARSER hid134,0);
}
static STATUS hid136(){
PARSER For_Optional_By, hid135;
return p_andparse0n (TOPARSER For_Optional_By,0,TOPARSER hid135,0);
}
static STATUS hid137(){
PARSER Expression, hid136;
return p_andparse0n (TOPARSER Expression,0,TOPARSER hid136,0);
}
static STATUS hid138(){
PARSER TO, hid137;
return p_andparse0n (TOPARSER TO,0,TOPARSER hid137,0);
}
static STATUS hid139(){
PARSER Expression, hid138;
return p_andparse0n (TOPARSER Expression,0,TOPARSER hid138,0);
}
static STATUS hid140(){
PARSER COLON_EQUALS, hid139;
return p_andparse0n (TOPARSER COLON_EQUALS,0,TOPARSER hid139,0);
}
static STATUS hid141(){
PARSER Ident, hid140;
return p_andparse0n (TOPARSER Ident,0,TOPARSER hid140,0);
}
static STATUS hid142(){
PARSER For_Start, hid141;
return p_andparse0n (TOPARSER For_Start,0,TOPARSER hid141,0);
}
STATUS For(){
PARSER hid142;
return((TOPARSER hid142)());
}


STATUS For_End(){
PARSER END;
return((TOPARSER END)());
}

static STATUS hid143(){
PARSER BY, Constant_Expression;
return p_andparse0n (TOPARSER BY,0,TOPARSER Constant_Expression,0);
}
static STATUS hid144(){
PARSER hid143;
return p_option0n(TOPARSER hid143,0);
}
STATUS For_Optional_By(){
PARSER hid144;
return((TOPARSER hid144)());
}


STATUS For_Start(){
PARSER FOR;
return((TOPARSER FOR)());
}




static STATUS hid145(){
PARSER Var_Assignment, Type;
return p_andparse0n (TOPARSER Var_Assignment,0,TOPARSER Type,0);
}
static STATUS hid146(){
PARSER Formal_Param_Section_Idents, hid145;
return p_andparse0n (TOPARSER Formal_Param_Section_Idents,0,TOPARSER hid145,0);
}
static STATUS hid147(){
PARSER Formal_Param_Section_Optional_Var, hid146;
return p_andparse0n (TOPARSER Formal_Param_Section_Optional_Var,0,TOPARSER hid146,0);
}
STATUS Formal_Param_Section(){
PARSER hid147;
return((TOPARSER hid147)());
}


static STATUS hid148(){
PARSER VAR;
return p_option0n(TOPARSER VAR,0);
}
STATUS Formal_Param_Section_Optional_Var(){
PARSER hid148;
return((TOPARSER hid148)());
}


static STATUS hid149(){
PARSER Formal_Param_Section_Ident_Separator, Ident;
return p_andparse0n (TOPARSER Formal_Param_Section_Ident_Separator,0,TOPARSER Ident,0);
}
static STATUS hid150(){
PARSER hid149;
return p_many0n(TOPARSER hid149,0);
}
static STATUS hid151(){
PARSER Ident, hid150;
return p_andparse0n (TOPARSER Ident,0,TOPARSER hid150,0);
}
STATUS Formal_Param_Section_Idents(){
PARSER hid151;
return((TOPARSER hid151)());
}


STATUS Formal_Param_Section_Ident_Separator(){
PARSER COMMA;
return((TOPARSER COMMA)());
}



static STATUS hid152(){
PARSER Formal_Param_Section_List;
return p_option0n(TOPARSER Formal_Param_Section_List,0);
}
STATUS Formal_Param_Sections(){
PARSER hid152;
return((TOPARSER hid152)());
}


static STATUS hid153(){
PARSER Formal_Param_Section_Separator, Formal_Param_Section;
return p_andparse0n (TOPARSER Formal_Param_Section_Separator,0,TOPARSER Formal_Param_Section,0);
}
static STATUS hid154(){
PARSER hid153;
return p_many0n(TOPARSER hid153,0);
}
static STATUS hid155(){
PARSER Formal_Param_Section, hid154;
return p_andparse0n (TOPARSER Formal_Param_Section,0,TOPARSER hid154,0);
}
STATUS Formal_Param_Section_List(){
PARSER hid155;
return((TOPARSER hid155)());
}


STATUS Formal_Param_Section_Separator(){
PARSER SEMI_COLON;
return((TOPARSER SEMI_COLON)());
}


static STATUS hid156(){
PARSER Formal_Params_End, Formal_Params_Optional_Type;
return p_andparse0n (TOPARSER Formal_Params_End,0,TOPARSER Formal_Params_Optional_Type,0);
}
static STATUS hid157(){
PARSER Formal_Param_Sections, hid156;
return p_andparse0n (TOPARSER Formal_Param_Sections,0,TOPARSER hid156,0);
}
static STATUS hid158(){
PARSER Formal_Params_Start, hid157;
return p_andparse0n (TOPARSER Formal_Params_Start,0,TOPARSER hid157,0);
}
STATUS Formal_Params(){
PARSER hid158;
return((TOPARSER hid158)());
}


static STATUS hid159(){
PARSER COLON, Type_Reference;
return p_andparse0n (TOPARSER COLON,0,TOPARSER Type_Reference,0);
}
static STATUS hid160(){
PARSER hid159;
return p_option0n(TOPARSER hid159,0);
}
STATUS Formal_Params_Optional_Type(){
PARSER hid160;
return((TOPARSER hid160)());
}


STATUS Formal_Params_Start(){
PARSER OPEN_PAREN;
return((TOPARSER OPEN_PAREN)());
}


STATUS Formal_Params_End(){
PARSER CLOSE_PAREN;
return((TOPARSER CLOSE_PAREN)());
}




static STATUS hid161(){
PARSER Procedure_Declaration_Optional_Formal_Params, Declaration_Procedure_Terminator;
return p_andparse0n (TOPARSER Procedure_Declaration_Optional_Formal_Params,0,TOPARSER Declaration_Procedure_Terminator,0);
}
static STATUS hid162(){
PARSER Procedure_Declaration_Name, hid161;
return p_andparse0n (TOPARSER Procedure_Declaration_Name,0,TOPARSER hid161,0);
}
static STATUS hid163(){
PARSER Procedure_Declaration_Optional_Receiver, hid162;
return p_andparse0n (TOPARSER Procedure_Declaration_Optional_Receiver,0,TOPARSER hid162,0);
}
static STATUS hid164(){
PARSER CARET, hid163;
return p_andparse0n (TOPARSER CARET,0,TOPARSER hid163,0);
}
static STATUS hid165(){
PARSER Procedure_Declaration_Start, hid164;
return p_andparse0n (TOPARSER Procedure_Declaration_Start,0,TOPARSER hid164,0);
}
STATUS Forward_Declaration(){
PARSER hid165;
return((TOPARSER hid165)());
}




static STATUS hid166(){
PARSER COLON, Type_Reference;
return p_andparse0n (TOPARSER COLON,0,TOPARSER Type_Reference,0);
}
static STATUS hid167(){
PARSER Qualified_Ident, hid166;
return p_andparse0n (TOPARSER Qualified_Ident,0,TOPARSER hid166,0);
}
STATUS Guard(){
PARSER hid167;
return((TOPARSER hid167)());
}



static STATUS hid168 (){
PARSER Ident_Export, Ident_ReadOnly_Export;
return p_orparse0n (TOPARSER Ident_Export,0,TOPARSER Ident_ReadOnly_Export,0);
}
static STATUS hid169(){
PARSER hid168;
return p_option0n(TOPARSER hid168,0);
}
static STATUS hid170(){
PARSER Ident, hid169;
return p_andparse0n (TOPARSER Ident,0,TOPARSER hid169,0);
}
STATUS Ident_Def(){
PARSER hid170;
return((TOPARSER hid170)());
}


STATUS Ident(){
PARSER IDENTIFIER;
return((TOPARSER IDENTIFIER)());
}



static STATUS hid171(){
PARSER Ident_Separator, Ident_Def;
return p_andparse0n (TOPARSER Ident_Separator,0,TOPARSER Ident_Def,0);
}
static STATUS hid172(){
PARSER hid171;
return p_many0n(TOPARSER hid171,0);
}
static STATUS hid173(){
PARSER Ident_Def, hid172;
return p_andparse0n (TOPARSER Ident_Def,0,TOPARSER hid172,0);
}
STATUS Ident_List(){
PARSER hid173;
return((TOPARSER hid173)());
}



STATUS Ident_Export(){
PARSER STAR;
return((TOPARSER STAR)());
}

STATUS Ident_ReadOnly_Export(){
PARSER HYPHEN;
return((TOPARSER HYPHEN)());
}


STATUS Ident_Separator(){
PARSER COMMA;
return((TOPARSER COMMA)());
}


/* Stephen says ... */

/* A rather large version of If.
** This has been expanded out somewhat to make it
** easier to construct an abstract syntax tree a
*/

/* End of what Stephen said - PTB */


static STATUS hid174(){
PARSER If_Else;
return p_option0n(TOPARSER If_Else,0);
}
static STATUS hid175(){
PARSER hid174, END;
return p_andparse0n (TOPARSER hid174,0,TOPARSER END,0);
}
static STATUS hid176 (){
PARSER If_Else_Ifs, hid175;
return p_orparse0n (TOPARSER If_Else_Ifs,0,TOPARSER hid175,0);
}
static STATUS hid177(){
PARSER If_Then, hid176;
return p_andparse0n (TOPARSER If_Then,0,TOPARSER hid176,0);
}
static STATUS hid178(){
PARSER Expression, hid177;
return p_andparse0n (TOPARSER Expression,0,TOPARSER hid177,0);
}
static STATUS hid179(){
PARSER IF, hid178;
return p_andparse0n (TOPARSER IF,0,TOPARSER hid178,0);
}
STATUS If(){
PARSER hid179;
return((TOPARSER hid179)());
}

/* that was the recursive way. Now lets be more straightforward - PTB */


static STATUS hid180(){
PARSER ELSE, Statement_Sequence;
return p_andparse0n (TOPARSER ELSE,0,TOPARSER Statement_Sequence,0);
}
STATUS If_Else(){
PARSER hid180;
return((TOPARSER hid180)());
}

/* Stephen says ... */

/*
** This is right recursive instead of the normal left recursive
** so that the elseif chain will be build up in the correct order.
** A left recursive rule would require reversing the links once the
** list was complete.  Does anybody have a convincing argument why
** this should be written in a left recursive manner and then list
** reversal done? (except for that fact that its easier to write a
** reverse that doesn't have a fixed limit like the YACC stack does)
*/

/* End of what Stephen said - PTB */

static STATUS hid181(){
PARSER Expression, If_Then;
return p_andparse0n (TOPARSER Expression,0,TOPARSER If_Then,0);
}
static STATUS hid182(){
PARSER ELSIF, hid181;
return p_andparse0n (TOPARSER ELSIF,0,TOPARSER hid181,0);
}
static STATUS hid183(){
PARSER hid182;
return p_some0n(TOPARSER hid182,0);
}
static STATUS hid184(){
PARSER If_Else;
return p_option0n(TOPARSER If_Else,0);
}
static STATUS hid185(){
PARSER hid184, END;
return p_andparse0n (TOPARSER hid184,0,TOPARSER END,0);
}
static STATUS hid186(){
PARSER hid183, hid185;
return p_andparse0n (TOPARSER hid183,0,TOPARSER hid185,0);
}
STATUS If_Else_Ifs(){
PARSER hid186;
return((TOPARSER hid186)());
}

/* NB - I didn't see any need for that recursion, so I took it out,
 * but one could try
 *  ELSIF Expression If_Then { If_Else_Ifs | [ If_Else ] END }
 *  - PTB */


static STATUS hid187(){
PARSER THEN, Statement_Sequence;
return p_andparse0n (TOPARSER THEN,0,TOPARSER Statement_Sequence,0);
}
STATUS If_Then(){
PARSER hid187;
return((TOPARSER hid187)());
}




static STATUS hid188(){
PARSER Import_List_Separator, Import_List_Import;
return p_andparse0n (TOPARSER Import_List_Separator,0,TOPARSER Import_List_Import,0);
}
static STATUS hid189(){
PARSER hid188;
return p_many0n(TOPARSER hid188,0);
}
static STATUS hid190(){
PARSER Import_List_Import, hid189;
return p_andparse0n (TOPARSER Import_List_Import,0,TOPARSER hid189,0);
}
STATUS Import_List(){
PARSER hid190;
return((TOPARSER hid190)());
}
       


static STATUS hid191(){
PARSER COLON_EQUALS, Ident;
return p_andparse0n (TOPARSER COLON_EQUALS,0,TOPARSER Ident,0);
}
static STATUS hid192(){
PARSER hid191;
return p_option0n(TOPARSER hid191,0);
}
static STATUS hid193(){
PARSER Ident, hid192;
return p_andparse0n (TOPARSER Ident,0,TOPARSER hid192,0);
}
STATUS Import_List_Import(){
PARSER hid193;
return((TOPARSER hid193)());
}


static STATUS hid194(){
PARSER Import_List, Statement_Separator;
return p_andparse0n (TOPARSER Import_List,0,TOPARSER Statement_Separator,0);
}
static STATUS hid195(){
PARSER IMPORT, hid194;
return p_andparse0n (TOPARSER IMPORT,0,TOPARSER hid194,0);
}
static STATUS hid196(){
PARSER hid195;
return p_option0n(TOPARSER hid195,0);
}
STATUS Import_List_Optional(){
PARSER hid196;
return((TOPARSER hid196)());
}



STATUS Import_List_Separator(){
PARSER COMMA;
return((TOPARSER COMMA)());
}




static STATUS hid197(){
PARSER Statement_Sequence, Loop_End;
return p_andparse0n (TOPARSER Statement_Sequence,0,TOPARSER Loop_End,0);
}
static STATUS hid198(){
PARSER Loop_Start, hid197;
return p_andparse0n (TOPARSER Loop_Start,0,TOPARSER hid197,0);
}
STATUS Loop(){
PARSER hid198;
return((TOPARSER hid198)());
}


STATUS Loop_End(){
PARSER END;
return((TOPARSER END)());
}


STATUS Loop_Start(){
PARSER LOOP;
return((TOPARSER LOOP)());
}




static STATUS hid199(){
PARSER Module_Name_Def, Module_End;
return p_andparse0n (TOPARSER Module_Name_Def,0,TOPARSER Module_End,0);
}
static STATUS hid200(){
PARSER Module_End_Block, hid199;
return p_andparse0n (TOPARSER Module_End_Block,0,TOPARSER hid199,0);
}
static STATUS hid201(){
PARSER Module_Optional_Begin_Block, hid200;
return p_andparse0n (TOPARSER Module_Optional_Begin_Block,0,TOPARSER hid200,0);
}
static STATUS hid202(){
PARSER Declaration_Sequence, hid201;
return p_andparse0n (TOPARSER Declaration_Sequence,0,TOPARSER hid201,0);
}
static STATUS hid203(){
PARSER Import_List_Optional, hid202;
return p_andparse0n (TOPARSER Import_List_Optional,0,TOPARSER hid202,0);
}
static STATUS hid204(){
PARSER Statement_Separator, hid203;
return p_andparse0n (TOPARSER Statement_Separator,0,TOPARSER hid203,0);
}
static STATUS hid205(){
PARSER Module_Name_Def, hid204;
return p_andparse0n (TOPARSER Module_Name_Def,0,TOPARSER hid204,0);
}
static STATUS hid206(){
PARSER Module_Start, hid205;
return p_andparse0n (TOPARSER Module_Start,0,TOPARSER hid205,0);
}
STATUS Module(){
PARSER hid206;
return((TOPARSER hid206)());
}

/* This is my insertion, to control qualified idents more closely */

STATUS Module_Ident(){
PARSER IDENTIFIER;
return((TOPARSER IDENTIFIER)());
}

static STATUS hid207(){
PARSER BEGIN, Statement_Sequence;
return p_andparse0n (TOPARSER BEGIN,0,TOPARSER Statement_Sequence,0);
}
static STATUS hid208(){
PARSER hid207;
return p_option0n(TOPARSER hid207,0);
}
STATUS Module_Optional_Begin_Block(){
PARSER hid208;
return((TOPARSER hid208)());
}


STATUS Module_Name_Def(){
PARSER Ident_Def;
return((TOPARSER Ident_Def)());
}


STATUS Module_End(){
PARSER PERIOD;
return((TOPARSER PERIOD)());
}


STATUS Module_End_Block(){
PARSER END;
return((TOPARSER END)());
}


STATUS Module_Start(){
PARSER MODULE;
return((TOPARSER MODULE)());
}




static STATUS hid209 (){
PARSER Mul_Op_Modulus, Mul_Op_Conjunction;
return p_orparse0n (TOPARSER Mul_Op_Modulus,0,TOPARSER Mul_Op_Conjunction,0);
}
static STATUS hid210 (){
PARSER Mul_Op_Integer_Quotient, hid209;
return p_orparse0n (TOPARSER Mul_Op_Integer_Quotient,0,TOPARSER hid209,0);
}
static STATUS hid211 (){
PARSER Mul_Op_Real_Quotient, hid210;
return p_orparse0n (TOPARSER Mul_Op_Real_Quotient,0,TOPARSER hid210,0);
}
static STATUS hid212 (){
PARSER Mul_Op_Product, hid211;
return p_orparse0n (TOPARSER Mul_Op_Product,0,TOPARSER hid211,0);
}
STATUS Mul_Op(){
PARSER hid212;
return((TOPARSER hid212)());
}


STATUS Mul_Op_Product(){
PARSER STAR;
return((TOPARSER STAR)());
}


STATUS Mul_Op_Real_Quotient(){
PARSER FORWARD_SLASH;
return((TOPARSER FORWARD_SLASH)());
}


STATUS Mul_Op_Integer_Quotient(){
PARSER DIV;
return((TOPARSER DIV)());
}


STATUS Mul_Op_Modulus(){
PARSER MOD;
return((TOPARSER MOD)());
}


STATUS Mul_Op_Conjunction(){
PARSER AMPERSAND;
return((TOPARSER AMPERSAND)());
}




STATUS Number_Sum(){
PARSER PLUS;
return((TOPARSER PLUS)());
}


STATUS Number_Difference(){
PARSER HYPHEN;
return((TOPARSER HYPHEN)());
}




static STATUS hid213(){
PARSER ELSE, Statement_Sequence;
return p_andparse0n (TOPARSER ELSE,0,TOPARSER Statement_Sequence,0);
}
static STATUS hid214(){
PARSER hid213;
return p_option0n(TOPARSER hid213,0);
}
STATUS Optional_Else(){
PARSER hid214;
return((TOPARSER hid214)());
}



STATUS Procedure_Call(){
PARSER Designator;
return((TOPARSER Designator)());
}




static STATUS hid215(){
PARSER Procedure_Declaration_Name, Declaration_Procedure_Terminator;
return p_andparse0n (TOPARSER Procedure_Declaration_Name,0,TOPARSER Declaration_Procedure_Terminator,0);
}
static STATUS hid216(){
PARSER Procedure_Declaration_End_Block, hid215;
return p_andparse0n (TOPARSER Procedure_Declaration_End_Block,0,TOPARSER hid215,0);
}
static STATUS hid217(){
PARSER Procedure_Declaration_Optional_Begin_Block, hid216;
return p_andparse0n (TOPARSER Procedure_Declaration_Optional_Begin_Block,0,TOPARSER hid216,0);
}
static STATUS hid218(){
PARSER Declaration_Sequence, hid217;
return p_andparse0n (TOPARSER Declaration_Sequence,0,TOPARSER hid217,0);
}
static STATUS hid219(){
PARSER Procedure_Declaration_Header_Terminator, hid218;
return p_andparse0n (TOPARSER Procedure_Declaration_Header_Terminator,0,TOPARSER hid218,0);
}
static STATUS hid220(){
PARSER Procedure_Declaration_Optional_Formal_Params, hid219;
return p_andparse0n (TOPARSER Procedure_Declaration_Optional_Formal_Params,0,TOPARSER hid219,0);
}
static STATUS hid221(){
PARSER Procedure_Declaration_Name, hid220;
return p_andparse0n (TOPARSER Procedure_Declaration_Name,0,TOPARSER hid220,0);
}
static STATUS hid222(){
PARSER Procedure_Declaration_Optional_Receiver, hid221;
return p_andparse0n (TOPARSER Procedure_Declaration_Optional_Receiver,0,TOPARSER hid221,0);
}
static STATUS hid223(){
PARSER Procedure_Declaration_Start, hid222;
return p_andparse0n (TOPARSER Procedure_Declaration_Start,0,TOPARSER hid222,0);
}
STATUS Procedure_Declaration(){
PARSER hid223;
return((TOPARSER hid223)());
}


STATUS Procedure_Declaration_End_Block(){
PARSER END;
return((TOPARSER END)());
}


STATUS Procedure_Declaration_Header_Terminator(){
PARSER SEMI_COLON;
return((TOPARSER SEMI_COLON)());
}


static STATUS hid224(){
PARSER BEGIN, Statement_Sequence;
return p_andparse0n (TOPARSER BEGIN,0,TOPARSER Statement_Sequence,0);
}
static STATUS hid225(){
PARSER hid224;
return p_option0n(TOPARSER hid224,0);
}
STATUS Procedure_Declaration_Optional_Begin_Block(){
PARSER hid225;
return((TOPARSER hid225)());
}


static STATUS hid226(){
PARSER Formal_Params;
return p_option0n(TOPARSER Formal_Params,0);
}
STATUS Procedure_Declaration_Optional_Formal_Params(){
PARSER hid226;
return((TOPARSER hid226)());
}


static STATUS hid227(){
PARSER Receiver;
return p_option0n(TOPARSER Receiver,0);
}
STATUS Procedure_Declaration_Optional_Receiver(){
PARSER hid227;
return((TOPARSER hid227)());
}


STATUS Procedure_Declaration_Name(){
PARSER Ident_Def;
return((TOPARSER Ident_Def)());
}


STATUS Procedure_Declaration_Start(){
PARSER PROCEDURE;
return((TOPARSER PROCEDURE)());
}




static STATUS hid228(){
PARSER Module_Ident, Qualified_Ident_Separator;
return p_andparse0n (TOPARSER Module_Ident,0,TOPARSER Qualified_Ident_Separator,0);
}
static STATUS hid229(){
PARSER hid228;
return p_option0n(TOPARSER hid228,0);
}
static STATUS hid230(){
PARSER hid229, Ident;
return p_andparse0n (TOPARSER hid229,0,TOPARSER Ident,0);
}
STATUS Qualified_Ident(){
PARSER hid230;
return((TOPARSER hid230)());
}



STATUS Qualified_Ident_Separator(){
PARSER PERIOD;
return((TOPARSER PERIOD)());
}

static STATUS hid231(){
PARSER Module_Ident, Qualified_Ident_Separator;
return p_andparse0n (TOPARSER Module_Ident,0,TOPARSER Qualified_Ident_Separator,0);
}
static STATUS hid232(){
PARSER hid231;
return p_option0n(TOPARSER hid231,0);
}
static STATUS hid233(){
PARSER hid232, Type_Ident;
return p_andparse0n (TOPARSER hid232,0,TOPARSER Type_Ident,0);
}
STATUS Qualified_Type_Ident(){
PARSER hid233;
return((TOPARSER hid233)());
}




static STATUS hid234(){
PARSER Ident, Receiver_End;
return p_andparse0n (TOPARSER Ident,0,TOPARSER Receiver_End,0);
}
static STATUS hid235(){
PARSER Var_Assignment, hid234;
return p_andparse0n (TOPARSER Var_Assignment,0,TOPARSER hid234,0);
}
static STATUS hid236(){
PARSER Ident, hid235;
return p_andparse0n (TOPARSER Ident,0,TOPARSER hid235,0);
}
static STATUS hid237(){
PARSER Receiver_Optional_Var, hid236;
return p_andparse0n (TOPARSER Receiver_Optional_Var,0,TOPARSER hid236,0);
}
static STATUS hid238(){
PARSER Receiver_Start, hid237;
return p_andparse0n (TOPARSER Receiver_Start,0,TOPARSER hid237,0);
}
STATUS Receiver(){
PARSER hid238;
return((TOPARSER hid238)());
}


STATUS Receiver_End(){
PARSER CLOSE_PAREN;
return((TOPARSER CLOSE_PAREN)());
}



static STATUS hid239(){
PARSER VAR;
return p_option0n(TOPARSER VAR,0);
}
STATUS Receiver_Optional_Var(){
PARSER hid239;
return((TOPARSER hid239)());
}


STATUS Receiver_Start(){
PARSER OPEN_PAREN;
return((TOPARSER OPEN_PAREN)());
}




static STATUS hid240 (){
PARSER Relation_Set_Membership, Relation_Type_Test;
return p_orparse0n (TOPARSER Relation_Set_Membership,0,TOPARSER Relation_Type_Test,0);
}
static STATUS hid241 (){
PARSER Relation_Greater_Equal_To, hid240;
return p_orparse0n (TOPARSER Relation_Greater_Equal_To,0,TOPARSER hid240,0);
}
static STATUS hid242 (){
PARSER Relation_Greater_Than, hid241;
return p_orparse0n (TOPARSER Relation_Greater_Than,0,TOPARSER hid241,0);
}
static STATUS hid243 (){
PARSER Relation_Less_Equal_To, hid242;
return p_orparse0n (TOPARSER Relation_Less_Equal_To,0,TOPARSER hid242,0);
}
static STATUS hid244 (){
PARSER Relation_Less_Than, hid243;
return p_orparse0n (TOPARSER Relation_Less_Than,0,TOPARSER hid243,0);
}
static STATUS hid245 (){
PARSER Relation_Not_Equal, hid244;
return p_orparse0n (TOPARSER Relation_Not_Equal,0,TOPARSER hid244,0);
}
static STATUS hid246 (){
PARSER Relation_Equal, hid245;
return p_orparse0n (TOPARSER Relation_Equal,0,TOPARSER hid245,0);
}
STATUS Relation(){
PARSER hid246;
return((TOPARSER hid246)());
}

STATUS Relation_Equal(){
PARSER EQUALS;
return((TOPARSER EQUALS)());
}

STATUS Relation_Greater_Than(){
PARSER RIGHT_CHEVRON;
return((TOPARSER RIGHT_CHEVRON)());
}

STATUS Relation_Greater_Equal_To(){
PARSER RIGHT_CHEVRON_EQUALS;
return((TOPARSER RIGHT_CHEVRON_EQUALS)());
}

STATUS Relation_Less_Than(){
PARSER LEFT_CHEVRON;
return((TOPARSER LEFT_CHEVRON)());
}

STATUS Relation_Not_Equal(){
PARSER HASH;
return((TOPARSER HASH)());
}

STATUS Relation_Less_Equal_To(){
PARSER LEFT_CHEVRON_EQUALS;
return((TOPARSER LEFT_CHEVRON_EQUALS)());
}

STATUS Relation_Set_Membership(){
PARSER IN;
return((TOPARSER IN)());
}

STATUS Relation_Type_Test(){
PARSER IS;
return((TOPARSER IS)());
}




static STATUS hid247(){
PARSER Repeat_Until, Expression;
return p_andparse0n (TOPARSER Repeat_Until,0,TOPARSER Expression,0);
}
static STATUS hid248(){
PARSER Statement_Sequence, hid247;
return p_andparse0n (TOPARSER Statement_Sequence,0,TOPARSER hid247,0);
}
static STATUS hid249(){
PARSER Repeat_Start, hid248;
return p_andparse0n (TOPARSER Repeat_Start,0,TOPARSER hid248,0);
}
STATUS Repeat(){
PARSER hid249;
return((TOPARSER hid249)());
}


STATUS Repeat_Until(){
PARSER UNTIL;
return((TOPARSER UNTIL)());
}


STATUS Repeat_Start(){
PARSER REPEAT;
return((TOPARSER REPEAT)());
}




static STATUS hid250(){
PARSER RETURN, Return_Optional_Expression;
return p_andparse0n (TOPARSER RETURN,0,TOPARSER Return_Optional_Expression,0);
}
STATUS Return(){
PARSER hid250;
return((TOPARSER hid250)());
}


static STATUS hid251(){
PARSER Expression;
return p_option0n(TOPARSER Expression,0);
}
STATUS Return_Optional_Expression(){
PARSER hid251;
return((TOPARSER hid251)());
}




static STATUS hid252(){
PARSER Set_Element_List;
return p_option0n(TOPARSER Set_Element_List,0);
}
static STATUS hid253(){
PARSER hid252, Set_End;
return p_andparse0n (TOPARSER hid252,0,TOPARSER Set_End,0);
}
static STATUS hid254(){
PARSER Set_Start, hid253;
return p_andparse0n (TOPARSER Set_Start,0,TOPARSER hid253,0);
}
STATUS Set(){
PARSER hid254;
return((TOPARSER hid254)());
}


static STATUS hid255(){
PARSER Set_Element_Separator, Expression;
return p_andparse0n (TOPARSER Set_Element_Separator,0,TOPARSER Expression,0);
}
static STATUS hid256(){
PARSER hid255;
return p_option0n(TOPARSER hid255,0);
}
static STATUS hid257(){
PARSER Expression, hid256;
return p_andparse0n (TOPARSER Expression,0,TOPARSER hid256,0);
}
STATUS Set_Element(){
PARSER hid257;
return((TOPARSER hid257)());
}


static STATUS hid258(){
PARSER Set_Element_List_Separator, Set_Element;
return p_andparse0n (TOPARSER Set_Element_List_Separator,0,TOPARSER Set_Element,0);
}
static STATUS hid259(){
PARSER hid258;
return p_many0n(TOPARSER hid258,0);
}
static STATUS hid260(){
PARSER Set_Element, hid259;
return p_andparse0n (TOPARSER Set_Element,0,TOPARSER hid259,0);
}
STATUS Set_Element_List(){
PARSER hid260;
return((TOPARSER hid260)());
}

STATUS Set_Element_List_Separator(){
PARSER COMMA;
return((TOPARSER COMMA)());
}

STATUS Set_Element_Separator(){
PARSER PERIOD_PERIOD;
return((TOPARSER PERIOD_PERIOD)());
}

STATUS Set_End(){
PARSER CLOSE_BRACE;
return((TOPARSER CLOSE_BRACE)());
}

STATUS Set_Start(){
PARSER OPEN_BRACE;
return((TOPARSER OPEN_BRACE)());
}



/* this next could be shorter, bt I'll leave it - PTB */

static STATUS hid261(){
PARSER Add_Op, Simple_Expression_Terms;
return p_andparse0n (TOPARSER Add_Op,0,TOPARSER Simple_Expression_Terms,0);
}
static STATUS hid262(){
PARSER hid261;
return p_option0n(TOPARSER hid261,0);
}
static STATUS hid263(){
PARSER Term, hid262;
return p_andparse0n (TOPARSER Term,0,TOPARSER hid262,0);
}
static STATUS hid264(){
PARSER Simple_Expression_Optional_Start, hid263;
return p_andparse0n (TOPARSER Simple_Expression_Optional_Start,0,TOPARSER hid263,0);
}
STATUS Simple_Expression(){
PARSER hid264;
return((TOPARSER hid264)());
}


static STATUS hid265(){
PARSER Add_Op, Term;
return p_andparse0n (TOPARSER Add_Op,0,TOPARSER Term,0);
}
static STATUS hid266(){
PARSER hid265;
return p_many0n(TOPARSER hid265,0);
}
static STATUS hid267(){
PARSER Term, hid266;
return p_andparse0n (TOPARSER Term,0,TOPARSER hid266,0);
}
STATUS Simple_Expression_Terms(){
PARSER hid267;
return((TOPARSER hid267)());
}



static STATUS hid268 (){
PARSER Simple_Expression_Negative, Simple_Expression_Positive;
return p_orparse0n (TOPARSER Simple_Expression_Negative,0,TOPARSER Simple_Expression_Positive,0);
}
static STATUS hid269(){
PARSER hid268;
return p_option0n(TOPARSER hid268,0);
}
STATUS Simple_Expression_Optional_Start(){
PARSER hid269;
return((TOPARSER hid269)());
}


STATUS Simple_Expression_Negative(){
PARSER HYPHEN;
return((TOPARSER HYPHEN)());
}

STATUS Simple_Expression_Positive(){
PARSER PLUS;
return((TOPARSER PLUS)());
}



static STATUS hid270 (){
PARSER Return, p_nothing0;
return p_orparse0n (TOPARSER Return,0,TOPARSER p_nothing0,0);
}
static STATUS hid271 (){
PARSER Exit, hid270;
return p_orparse0n (TOPARSER Exit,0,TOPARSER hid270,0);
}
static STATUS hid272 (){
PARSER With, hid271;
return p_orparse0n (TOPARSER With,0,TOPARSER hid271,0);
}
static STATUS hid273 (){
PARSER Loop, hid272;
return p_orparse0n (TOPARSER Loop,0,TOPARSER hid272,0);
}
static STATUS hid274 (){
PARSER For, hid273;
return p_orparse0n (TOPARSER For,0,TOPARSER hid273,0);
}
static STATUS hid275 (){
PARSER Repeat, hid274;
return p_orparse0n (TOPARSER Repeat,0,TOPARSER hid274,0);
}
static STATUS hid276 (){
PARSER While, hid275;
return p_orparse0n (TOPARSER While,0,TOPARSER hid275,0);
}
static STATUS hid277 (){
PARSER Case, hid276;
return p_orparse0n (TOPARSER Case,0,TOPARSER hid276,0);
}
static STATUS hid278 (){
PARSER If, hid277;
return p_orparse0n (TOPARSER If,0,TOPARSER hid277,0);
}
static STATUS hid279 (){
PARSER Procedure_Call, hid278;
return p_orparse0n (TOPARSER Procedure_Call,0,TOPARSER hid278,0);
}
static STATUS hid280 (){
PARSER Assignment, hid279;
return p_orparse0n (TOPARSER Assignment,0,TOPARSER hid279,0);
}
STATUS Statement(){
PARSER hid280;
return((TOPARSER hid280)());
}


STATUS Statement_Separator(){
PARSER SEMI_COLON;
return((TOPARSER SEMI_COLON)());
}

static STATUS hid281(){
PARSER Statement_Separator, Statement;
return p_andparse0n (TOPARSER Statement_Separator,0,TOPARSER Statement,0);
}
static STATUS hid282(){
PARSER hid281;
return p_many0n(TOPARSER hid281,0);
}
static STATUS hid283(){
PARSER Statement, hid282;
return p_andparse0n (TOPARSER Statement,0,TOPARSER hid282,0);
}
STATUS Statement_Sequence(){
PARSER hid283;
return((TOPARSER hid283)());
}




static STATUS hid284(){
PARSER Mul_Op, Factor;
return p_andparse0n (TOPARSER Mul_Op,0,TOPARSER Factor,0);
}
static STATUS hid285(){
PARSER hid284;
return p_many0n(TOPARSER hid284,0);
}
static STATUS hid286(){
PARSER Factor, hid285;
return p_andparse0n (TOPARSER Factor,0,TOPARSER hid285,0);
}
STATUS Term(){
PARSER hid286;
return((TOPARSER hid286)());
}




static STATUS hid287 (){
PARSER Type_Pointer, Type_Procedure;
return p_orparse0n (TOPARSER Type_Pointer,0,TOPARSER Type_Procedure,0);
}
static STATUS hid288 (){
PARSER Type_Record, hid287;
return p_orparse0n (TOPARSER Type_Record,0,TOPARSER hid287,0);
}
static STATUS hid289 (){
PARSER Type_Array, hid288;
return p_orparse0n (TOPARSER Type_Array,0,TOPARSER hid288,0);
}
static STATUS hid290 (){
PARSER Type_Reference, hid289;
return p_orparse0n (TOPARSER Type_Reference,0,TOPARSER hid289,0);
}
STATUS Type(){
PARSER hid290;
return((TOPARSER hid290)());
}

static STATUS hid291(){
PARSER OF, Type;
return p_andparse0n (TOPARSER OF,0,TOPARSER Type,0);
}
static STATUS hid292(){
PARSER Type_Array_Bounds, hid291;
return p_andparse0n (TOPARSER Type_Array_Bounds,0,TOPARSER hid291,0);
}
static STATUS hid293(){
PARSER ARRAY, hid292;
return p_andparse0n (TOPARSER ARRAY,0,TOPARSER hid292,0);
}
STATUS Type_Array(){
PARSER hid293;
return((TOPARSER hid293)());
}


static STATUS hid294(){
PARSER Type_Array_Bounds_Separator, Constant_Expression;
return p_andparse0n (TOPARSER Type_Array_Bounds_Separator,0,TOPARSER Constant_Expression,0);
}
static STATUS hid295(){
PARSER hid294;
return p_many0n(TOPARSER hid294,0);
}
static STATUS hid296(){
PARSER Constant_Expression, hid295;
return p_andparse0n (TOPARSER Constant_Expression,0,TOPARSER hid295,0);
}
static STATUS hid297 (){
PARSER hid296, p_nothing0;
return p_orparse0n (TOPARSER hid296,0,TOPARSER p_nothing0,0);
}
STATUS Type_Array_Bounds(){
PARSER hid297;
return((TOPARSER hid297)());
}


STATUS Type_Array_Bounds_Separator(){
PARSER COMMA;
return((TOPARSER COMMA)());
}


STATUS Type_Assignment(){
PARSER EQUALS;
return((TOPARSER EQUALS)());
}


static STATUS hid298(){
PARSER Type, Type_Declaration_Terminator;
return p_andparse0n (TOPARSER Type,0,TOPARSER Type_Declaration_Terminator,0);
}
static STATUS hid299(){
PARSER Type_Assignment, hid298;
return p_andparse0n (TOPARSER Type_Assignment,0,TOPARSER hid298,0);
}
static STATUS hid300(){
PARSER Ident_Def, hid299;
return p_andparse0n (TOPARSER Ident_Def,0,TOPARSER hid299,0);
}
STATUS Type_Declaration(){
PARSER hid300;
return((TOPARSER hid300)());
}


static STATUS hid301(){
PARSER Type_Declaration;
return p_many0n(TOPARSER Type_Declaration,0);
}
STATUS Type_Declarations(){
PARSER hid301;
return((TOPARSER hid301)());
}

STATUS Type_Declaration_Terminator(){
PARSER SEMI_COLON;
return((TOPARSER SEMI_COLON)());
}

STATUS Type_Ident(){
PARSER IDENTIFIER;
return((TOPARSER IDENTIFIER)());
}

static STATUS hid302(){
PARSER TO, Type;
return p_andparse0n (TOPARSER TO,0,TOPARSER Type,0);
}
static STATUS hid303(){
PARSER POINTER, hid302;
return p_andparse0n (TOPARSER POINTER,0,TOPARSER hid302,0);
}
STATUS Type_Pointer(){
PARSER hid303;
return((TOPARSER hid303)());
}


static STATUS hid304(){
PARSER PROCEDURE, Procedure_Declaration_Optional_Formal_Params;
return p_andparse0n (TOPARSER PROCEDURE,0,TOPARSER Procedure_Declaration_Optional_Formal_Params,0);
}
STATUS Type_Procedure(){
PARSER hid304;
return((TOPARSER hid304)());
}

static STATUS hid305(){
PARSER Type_Record_Field_List, END;
return p_andparse0n (TOPARSER Type_Record_Field_List,0,TOPARSER END,0);
}
static STATUS hid306(){
PARSER Type_Record_Optional_Qualifier, hid305;
return p_andparse0n (TOPARSER Type_Record_Optional_Qualifier,0,TOPARSER hid305,0);
}
static STATUS hid307(){
PARSER RECORD, hid306;
return p_andparse0n (TOPARSER RECORD,0,TOPARSER hid306,0);
}
STATUS Type_Record(){
PARSER hid307;
return((TOPARSER hid307)());
}

static STATUS hid308(){
PARSER Type_Record_Field_Assignment, Type;
return p_andparse0n (TOPARSER Type_Record_Field_Assignment,0,TOPARSER Type,0);
}
static STATUS hid309(){
PARSER Ident_List, hid308;
return p_andparse0n (TOPARSER Ident_List,0,TOPARSER hid308,0);
}
static STATUS hid310(){
PARSER hid309;
return p_option0n(TOPARSER hid309,0);
}
STATUS Type_Record_Field(){
PARSER hid310;
return((TOPARSER hid310)());
}


STATUS Type_Record_Field_Assignment(){
PARSER COLON;
return((TOPARSER COLON)());
}


static STATUS hid311(){
PARSER Type_Record_Field_Separator, Type_Record_Field;
return p_andparse0n (TOPARSER Type_Record_Field_Separator,0,TOPARSER Type_Record_Field,0);
}
static STATUS hid312(){
PARSER hid311;
return p_many0n(TOPARSER hid311,0);
}
static STATUS hid313(){
PARSER Type_Record_Field, hid312;
return p_andparse0n (TOPARSER Type_Record_Field,0,TOPARSER hid312,0);
}
STATUS Type_Record_Field_List(){
PARSER hid313;
return((TOPARSER hid313)());
}


STATUS Type_Record_Field_Separator(){
PARSER SEMI_COLON;
return((TOPARSER SEMI_COLON)());
}


static STATUS hid314(){
PARSER Record_Base_Type;
return p_option0n(TOPARSER Record_Base_Type,0);
}
STATUS Type_Record_Optional_Qualifier(){
PARSER hid314;
return((TOPARSER hid314)());
}

static STATUS hid315(){
PARSER Type_Reference, CLOSE_PAREN;
return p_andparse0n (TOPARSER Type_Reference,0,TOPARSER CLOSE_PAREN,0);
}
static STATUS hid316(){
PARSER OPEN_PAREN, hid315;
return p_andparse0n (TOPARSER OPEN_PAREN,0,TOPARSER hid315,0);
}
STATUS Record_Base_Type(){
PARSER hid316;
return((TOPARSER hid316)());
}

/* Stephen says ... */

/* This is where I find that I have to distinguish type references from
   qualified idents. Idents are distinguished from keywords syntactically
   by containing a lower case letter, but that's too restrictive for their
   use as types, which can be, for example, INTEGER, and don't seem to
   be defined as keywords.

   What I have done is distinguish between qualified type idents and
   qualified (variable) idents, by demanding that the latter are of the form
   [a.]b, whilst the former may be looser and take the form [a.]B . The
   problem only arises when the module ident qualifier isn't there,
   because keywords like END may be mistaken as type idents, but I think
   that a type is always preceded by  a colon.
 */

/* End of what Stephen said - PTB */


STATUS Type_Reference(){
PARSER Qualified_Type_Ident;
return((TOPARSER Qualified_Type_Ident)());
}


STATUS Var_Assignment(){
PARSER COLON;
return((TOPARSER COLON)());
}


static STATUS hid317(){
PARSER Type, Var_Declaration_Terminator;
return p_andparse0n (TOPARSER Type,0,TOPARSER Var_Declaration_Terminator,0);
}
static STATUS hid318(){
PARSER Var_Assignment, hid317;
return p_andparse0n (TOPARSER Var_Assignment,0,TOPARSER hid317,0);
}
static STATUS hid319(){
PARSER Ident_List, hid318;
return p_andparse0n (TOPARSER Ident_List,0,TOPARSER hid318,0);
}
STATUS Var_Declaration(){
PARSER hid319;
return((TOPARSER hid319)());
}


static STATUS hid320(){
PARSER Var_Declaration;
return p_many0n(TOPARSER Var_Declaration,0);
}
STATUS Var_Declarations(){
PARSER hid320;
return((TOPARSER hid320)());
}


STATUS Var_Declaration_Terminator(){
PARSER SEMI_COLON;
return((TOPARSER SEMI_COLON)());
}



static STATUS hid321(){
PARSER Statement_Sequence, While_End;
return p_andparse0n (TOPARSER Statement_Sequence,0,TOPARSER While_End,0);
}
static STATUS hid322(){
PARSER DO, hid321;
return p_andparse0n (TOPARSER DO,0,TOPARSER hid321,0);
}
static STATUS hid323(){
PARSER Expression, hid322;
return p_andparse0n (TOPARSER Expression,0,TOPARSER hid322,0);
}
static STATUS hid324(){
PARSER While_Start, hid323;
return p_andparse0n (TOPARSER While_Start,0,TOPARSER hid323,0);
}
STATUS While(){
PARSER hid324;
return((TOPARSER hid324)());
}


STATUS While_End(){
PARSER END;
return((TOPARSER END)());
}


STATUS While_Start(){
PARSER WHILE;
return((TOPARSER WHILE)());
}


static STATUS hid325(){
PARSER Optional_Else, With_End;
return p_andparse0n (TOPARSER Optional_Else,0,TOPARSER With_End,0);
}
static STATUS hid326(){
PARSER With_Statements, hid325;
return p_andparse0n (TOPARSER With_Statements,0,TOPARSER hid325,0);
}
static STATUS hid327(){
PARSER With_Start, hid326;
return p_andparse0n (TOPARSER With_Start,0,TOPARSER hid326,0);
}
STATUS With(){
PARSER hid327;
return((TOPARSER hid327)());
}


STATUS With_End(){
PARSER END;
return((TOPARSER END)());
}


STATUS With_Start(){
PARSER WITH;
return((TOPARSER WITH)());
}


static STATUS hid328(){
PARSER DO, Statement_Sequence;
return p_andparse0n (TOPARSER DO,0,TOPARSER Statement_Sequence,0);
}
static STATUS hid329(){
PARSER Guard, hid328;
return p_andparse0n (TOPARSER Guard,0,TOPARSER hid328,0);
}
STATUS With_Statement(){
PARSER hid329;
return((TOPARSER hid329)());
}


static STATUS hid330(){
PARSER With_Statements_Separator, With_Statement;
return p_andparse0n (TOPARSER With_Statements_Separator,0,TOPARSER With_Statement,0);
}
static STATUS hid331(){
PARSER hid330;
return p_many0n(TOPARSER hid330,0);
}
static STATUS hid332(){
PARSER With_Statement, hid331;
return p_andparse0n (TOPARSER With_Statement,0,TOPARSER hid331,0);
}
STATUS With_Statements(){
PARSER hid332;
return((TOPARSER hid332)());
}


STATUS With_Statements_Separator(){
PARSER VERTICAL_BAR;
return((TOPARSER VERTICAL_BAR)());
}

/* End of script - PTB */

